﻿using System;
using System.Threading;
using Icc4cExample;
using ICC4cPwmSdk.Device;
using ICC4cPwmSdk.Dtos;

namespace Icc1cExample
{
    internal class Icc1cExample
    {
        // To run this example, rename AnaInMain to Main and in Example.cs rename Main to ExMain for example
        // static void Main(string[] args) -> is always an entry point
        static void Icc1cMain(string[] args)
        {
            Console.WriteLine("Starting basic ICC-1C example...");
            Icc1cSdkDeviceController controller;

            while (true)
            {
                Console.WriteLine("Select connection: C = COM, E = ETHERNET");
                var key = Console.ReadKey();
                Console.WriteLine();
                //COM connection
                if (key.Key == ConsoleKey.C)
                {
                    controller = new Icc1cSdkDeviceController();
                    Console.WriteLine("What is the COM number of the ICC-1c?");
                    var comport = Console.ReadLine();
                    if (int.TryParse(comport, out var comInt))
                    {
                        if (controller.Connect(comInt))
                        {
                            break;
                        }

                        Console.WriteLine("Connection failed.");
                    }
                }

                // Ethernet connection
                if (key.Key == ConsoleKey.E)
                {

                    controller = new Icc1cSdkDeviceController(true);
                    Console.WriteLine("Static IP or automatic search? A = auto, S = static");
                    key = Console.ReadKey();
                    Console.WriteLine();

                    //Automatic ethernet search (DHCP has to be enabled in the board EEPROM (default))
                    if (key.Key == ConsoleKey.A)
                    {
                        var devices = EthernetSearcher.SearchEthernetIccBoards();
                        if (devices.Count == 0)
                        {
                            Console.WriteLine("No ethernet boards found.");
                        }
                        else
                        {
                            Console.WriteLine($"Boards found({devices.Count}):");

                            var index = 0;
                            foreach (var device in devices)
                            {
                                Console.WriteLine($"Index: {index}, Serial number: {device.SerialNumber}, IP: {device.Ip}, Port: {device.Port}");
                                index++;
                            }
                            Console.WriteLine("Select index of the device you want to connect.");
                            var selectedIndex = Console.ReadLine();
                            if (int.TryParse(selectedIndex, out var indexInt) && indexInt >= 0 && indexInt < devices.Count)
                            {
                                if (controller.Connect(devices[indexInt].Ip, devices[indexInt].Port))
                                {
                                    break;
                                }
                                Console.WriteLine("Connection failed.");
                            }
                        }

                    }

                    //Static IP connection (static IP must be configured in the board EEPROM (using Optotune Cockpit application), also DHCP has to be switched of in the board EEPROM)
                    if (key.Key == ConsoleKey.S)
                    {
                        Console.WriteLine("Specify IP address of the board:");
                        var ipAddress = Console.ReadLine();

                        if (controller.Connect(ipAddress, 5000))
                        {
                            break;
                        }
                        Console.WriteLine("Connection failed.");
                     }
                }
            }

            //Set board to PRO mode
            controller.ChangeModeToPro();

            //Getting board info
            Console.WriteLine("Board info");
            var serialNumber = controller.GetElectronicSerialNumber();
            var fwVersion =
                $"{controller.Status.GetBoardFirmwareMajorVersion()}.{controller.Status.GetBoardFirmwareMinorVersion()}.{controller.Status.GetBoardFirmwareRevisionVersion()}";

            Console.WriteLine($"Board serial number: {serialNumber}");
            Console.WriteLine($"Board firmware version: {fwVersion}");

            // In order to drive the connected product, device type of the product has to be obtained (this is not needed if
            // automatic detection is allowed)
            var connectedDevice = controller.GetDeviceType();
            Console.WriteLine();
            Console.WriteLine(
                $"Connected device: {string.Join(Enum.GetName(typeof(EConnectedDeviceType), connectedDevice))}");
            Console.WriteLine();

            var lensSerialNumber = controller.GetElectronicSerialNumber();
            Console.WriteLine(
                $"Lens {Enum.GetName(typeof(EConnectedDeviceType), connectedDevice)} ({lensSerialNumber}) found");

            var lensTemperature = controller.TemperatureManager.GetDeviceTemperature();

            Console.WriteLine($"Lens temperature: {lensTemperature}°C");

            var minCurrent = controller.DeviceEeprom.GetMaxNegativeCurrent();
            var maxCurrent = controller.DeviceEeprom.GetMaxPositiveCurrent();

            Console.WriteLine($"Minimum current is {minCurrent} mA, maximum current is {maxCurrent} mA");

            Console.WriteLine();
            Console.WriteLine("Setting static current");
            //Setting input system to static input
            controller.InputStage.ChangeActiveSystem(EInputSignalStageSystem.StaticInput);

            for (var current = minCurrent; current <= maxCurrent; current += (maxCurrent - minCurrent) / 5)
            {
                //Value has to be converted from mA to A
                var currentInA = current / 1000;
                Console.WriteLine($"Current {currentInA} A");
                controller.InputStage.StaticValue.SetCurrent(currentInA);
                //Diopter can be set similarly
                //controller.InputStage.StaticValue.SetDiopter(lensChannel, diopterInDpt);
                Thread.Sleep(1000);
            }

            Console.WriteLine("Setting static current to 0 A");
            controller.InputStage.StaticValue.SetCurrent(0);

            Console.WriteLine();
            Console.WriteLine("Running signal generator");

            controller.InputStage.ChangeActiveSystem(EInputSignalStageSystem.SignalGenerator);
            controller.InputStage.SignalGenerator.SetUnitType(EICC4cPwmUnitType.Current);
            controller.InputStage.SignalGenerator.SetShape(ESignalGeneratorShape.Sinusoidal);
            controller.InputStage.SignalGenerator.SetAmplitude(0.2f);
            controller.InputStage.SignalGenerator.SetFrequency(5);
            controller.InputStage.SignalGenerator.SwitchRunning(true);

            for (var i = 0; i < 5; i++)
            {
                Console.Write(".");
                Thread.Sleep(1000);
            }

            controller.InputStage.SignalGenerator.SwitchRunning(false);

            Console.WriteLine();
            Console.WriteLine("Signal generator stopped");

            Console.WriteLine();
            Console.WriteLine("Device EEPROM");

            var eepromVersion =
                $"{controller.DeviceEeprom.GetEepromVersion()}.{controller.DeviceEeprom.GetEepromSubVersion()}";
            Console.WriteLine($"EEPROM version: {eepromVersion}");

            var eepromBytes = controller.DeviceEeprom.GetBytes(0, 10);
            var eepromSize = controller.DeviceEeprom.GetDeviceEepromSize();

            Console.WriteLine(
                $"Printing {eepromBytes.Length}/{eepromSize} bytes saved in EEPROM: {string.Join(", ", eepromBytes)}");

            Console.WriteLine();
            Console.WriteLine("Example finished.");
            Console.ReadLine();
        }
    }
}